const internal = require("stream");
const { __glob, __web } = require("../modules/global-variables");
const { LogType } = require("./sub-log");
const log = require("./sub-log");
const auth = require("./sub-auth");
const cookieParser = require("cookie-parser");
const { error } = require("console");
const wlog = new LogType("Web")

module.exports.WebServer = class {

    constructor() {

        wlog.step.init("start_server", "Démarrage du serveur Express (Web)")
        init()
      
    }

}

function init() {
    const createError = require('http-errors');
    const express = require('express');
    const path = require('path');
    const cookieParser = require('cookie-parser');
    const http = require("http");

    const app = express();

    const port = normalizePort(process.env.PORT || '4000');


    const server = require('http').createServer(app);
    const io = require('socket.io')(server)

    const indexRouter = require(__web.ROUTER + "index.js");
    const loginRouter = require(__web.ROUTER + "login.js");
    const internalRouter = require(__web.ROUTER + "internal.js")

    IOConnection(io)
    
    
    app.set('views', __web.TEMPLATES); // general config
    app.set('view engine', 'ejs');
    app.use(express.json());
    app.use(express.urlencoded({ extended: false }));
    app.use(cookieParser());
    app.use(express.static(__web.PUBLIC));
    app.set('port', port);
    app.use('/', indexRouter);
    app.use('/login', loginRouter);
    app.use("/internal", internalRouter)


    app.use(function (req, res, next) {
        next(createError(404));
    });

    app.use(function (err, req, res, next) {
 
        // Set locals, only providing error
        // in development
        res.locals.message = err.message;
        res.locals.error = req.app.get('env')
            === 'development' ? err : {};
     
        // render the error page
        res.status(err.status || 500);
        res.render('error');
    });

    server.listen(port);
    server.on('error', onError(error, port));
    server.on('listening', () => {

        let addr = server.address();
        let bind = typeof addr === 'string'
            ? 'pipe ' + addr
            : 'port ' + addr.port;
        wlog.log("Serveur démarré sur le port : " + bind )
    });

    wlog.step.end("start_server")

}

// EXPRESS FUNCTIONS

function normalizePort(val) {
    let port = parseInt(val, 10);
 
    if (isNaN(port)) {
 
        // Named pipe
        return val;
    }
 
    if (port >= 0) {
 
        // Port number
        return port;
    }
 
    return false;
}
 
// Event listener for HTTP server "error" event
function onError(error, port) {
    if (error.syscall !== 'listen') {
        throw error;
    }
 
    let bind = typeof port === 'string'
        ? 'Pipe ' + port
        : 'Port ' + port;
 
    // Handle specific listen errors with
    // friendly messages
    switch (error.code) {
        case 'EACCES':
            console.error(bind
                + ' requires elevated privileges');
            process.exit(1);
            break;
        case 'EADDRINUSE':
            console.error(bind + ' is already in use');
            process.exit(1);
            break;
        default:
            throw error;
    }
}


function IOConnection(io) {

    const alog = log.getInstance("Authentification")

    io.on("connection", (socket) => {

        wlog.log("[SOCKET] - Nouvelle session : " + socket.id)

        socket.on("disconnect", () => {

            wlog.log("[SOCKET] - Fin de session : " + socket.id)
        })

        socket.on("GET/DISCORD_LOGIN_LINK", () => {


            var discordlink = null

            if(process.env.DEV == "true") {

                alog.log("Mode Developpeur Actif : Redirige vers LOCALHOST")
                discordlink = "https://discord.com/api/oauth2/authorize?client_id=1094727789682380922&redirect_uri=http%3A%2F%2Flocalhost%3A4000%2Finternal%2Fredirect&response_type=code&scope=identify%20guilds%20guilds.members.read" //DEV
            } else {
                discordlink = "https://discord.com/api/oauth2/authorize?client_id=1094727789682380922&redirect_uri=https%3A%2F%2Fsubsonics.raphix.fr%2Finternal%2Fredirect&response_type=code&scope=identify%20guilds%20guilds.members.read" //OFFICIEL
            }

            alog.log("Redirection de '" + socket.id + "' vers le service d'Authentification de Discord [ETAPE 1]")
            io.emit("ANSWER/GET/DISCORD_LOGIN_LINK", discordlink )
        })


        socket.on("GET/USER_INFO", () => {


            var token = socket.handshake.headers.cookie

            if(token) {

                token = socket.handshake.headers.cookie.replace("token=", "")
                 
                if(auth.checkUser(token)) {

                    const user = auth.getUser(token)
                    alog.log("Envoi des informations Discord de '" + user.user.username + "' à '" + socket.id + "'" )
                    socket.emit("ANSWER/GET/USER_INFO",user)
                
                } else {

                    io.emit("ANSWER/GET/USER_INFO", {"error":"USER_DONT_EXIST"})
                }
            } else {
                io.emit("ANSWER/GET/USER_INFO", {"error":"TOKEN_NOT_FINDED"})

            }
         
           

        })
    })

   

}